<?php

if (!defined('ABSPATH')) {
	exit;
}

/**
 * Create the admin page under wp-admin -> WooCommerce -> Casso
 *
 * @author   Casso Team
 * @since    
 *
 */
class Casso_Admin_Page
{

	/**
	 * @var string The message to display after saving settings
	 */
	var $message = '';
	/**
	 * casso_Admin_Page constructor.
	 */
	public function __construct()
	{

		$this->casso_get_list_banks =  CassoPayment::casso_get_list_banks();
		$this->casso_get_list_bin =  CassoPayment::casso_get_list_bin();
		$this->casso_get_status =  CassoPayment::casso_connect_status_banks();
		$this->settings = CassoPayment::get_settings();
		$this->oauth_settings = CassoPayment::casso_oauth_get_settings();
		if (isset($_REQUEST['oauth2_status'])) {
			$this->disconnectOAuth2();
		}
		if (isset($_REQUEST['casso_nonce']) && isset($_REQUEST['action']) && 'casso_save_settings' == $_REQUEST['action']) {
			$this->save_settings();
		}
		add_action('admin_menu', array($this, 'register_submenu_page'));
	}

	/**
	 * Save settings for the plugin
	 */
	public function save_settings()
	{

		if (wp_verify_nonce($_REQUEST['casso_nonce'], 'casso_save_settings')) {

			$_REQUEST['settings']['bank_transfer_accounts'] = $this->settings['bank_transfer_accounts'];

			if (strlen($this->settings['bank_transfer']['secure_token']) <= 0) {
				$_REQUEST['settings']['bank_transfer']['secure_token'] = CassoPayment::generate_random_string(16);
			} else {
				$_REQUEST['settings']['bank_transfer']['secure_token'] = $this->settings['bank_transfer']['secure_token'];
			}

			$temp = $_REQUEST['settings']['bank_transfer']['authorization_code_force_delete'];
			unset($_REQUEST['settings']['bank_transfer']['authorization_code_force_delete']);
			// Xoá kí tự đặc biệt và xóa bớt nếu dài quá, xóa khoảng trắng
			$_REQUEST['settings']['bank_transfer']['transaction_prefix'] = $this->clean_prefix($_REQUEST['settings']['bank_transfer']['transaction_prefix']);
			update_option('casso', $_REQUEST['settings']);
			$_REQUEST['settings']['bank_transfer']['authorization_code_force_delete'] = $temp;
			// xử lí webhook!
			$this->message = $this->casso_oauth_process_webhook($_REQUEST['settings']);
			// Message for use
			$this->message .=
				'<div class="updated notice"><p><strong>' .
				__('Settings saved', 'casso') .
				'</p></strong></div>';
		} else {

			$this->message =
				'<div class="error notice"><p><strong>' .
				__('Can not save settings! Please refresh this page.', 'casso') .
				'</p></strong></div>';
		}
	}
	public function console_log($output, $with_script_tags = true)
	{
		$js_code = 'console.log(' . json_encode($output, JSON_HEX_TAG) .
			');';
		if ($with_script_tags) {
			$js_code = '<script>' . $js_code . '</script>';
		}
		echo $js_code;
	}


	/**
	 * Register the sub-menu under "WooCommerce"
	 * Link: http://my-site.com/wp-admin/admin.php?page=casso
	 */
	public function register_submenu_page()
	{
		add_submenu_page(
			'woocommerce',
			__('Casso Settings', 'casso'),
			'Casso',
			'manage_options',
			'casso',
			array($this, 'admin_page_html')
		);
	}

	/**
	 * Generate the HTML code of the settings page
	 */
	public function admin_page_html()
	{
		$urlConnectOauth = CassoPayment::$connect_uri . '/oauth2-login';
		$dataQuery = base64_encode(json_encode(array(
			"redirect_uri" =>  WC()->api_request_url(CassoPayment::$webhook_oauth2),
			"client_id" => CassoPayment::$client_id,
		)));
		$urlConnectOauth .= '?code=' . $dataQuery;


		// check user capabilities
		if (!current_user_can('manage_options')) {
			return;
		}
		$settings = CassoPayment::get_settings();
?>
		<div class="wrap">
			<h1><?= esc_html(get_admin_page_title()); ?></h1>
			<form name="woocommerce_for_vietnam" method="post">
				<?php echo $this->message ?>
				<input type="hidden" id="action" name="action" value="casso_save_settings">
				<input type="hidden" id="casso_nonce" name="casso_nonce" value="<?php echo wp_create_nonce('casso_save_settings') ?>">
				<table class="form-table">
					<tbody>
						<tr>
							<th scope="row"><?php echo __('Enable/Disable', 'casso-wordpress-plugin'); ?></th>
							<td>
								<input name="settings[bank_transfer][enabled]" type="hidden" value="no">
								<input name="settings[bank_transfer][enabled]" type="checkbox" id="bank_transfer" value="yes" <?php if ('yes' == $settings['bank_transfer']['enabled'])
																																	echo 'checked="checked"' ?>>
								<label for="bank_transfer" style="font-size: 13px; font-style: oblique;"><?php echo __('Turn on bank transfer', 'casso-wordpress-plugin'); ?></label>
								<br />
							</td>
						</tr>
						<tr>
							<th scope="row"><?php echo __('VietQR', 'casso-wordpress-plugin'); ?></th>
							<td>
								<input name="settings[bank_transfer][viet_qr]" type="hidden" value="no">
								<input name="settings[bank_transfer][viet_qr]" type="checkbox" id="bank_transfer" value="yes" <?php if (!empty($settings['bank_transfer']['viet_qr']) && 'yes' == $settings['bank_transfer']['viet_qr'])
																																	echo 'checked="checked"' ?>>
								<label for="bank_transfer" style="font-size: 13px; font-style: oblique;"><?php echo __('Enable QR code display mode VietQR', 'casso-wordpress-plugin'); ?></label>
								<br />
							</td>
						</tr>
						<tr style="<?php if ($this->oauth_settings['login_type'] == 1) echo "display: none;" ?>">
							<th scope="row"><?php echo __('Link Casso', 'casso-wordpress-plugin') ?></th><?php echo __('Set up a link with Casso', 'casso-wordpress-plugin'); ?>
							<td>
								<a type="button" href="<?php echo $urlConnectOauth; ?>" style="border: none; background-color: #056fca; color: white; border-radius: 5px; padding: 6px; min-width: 210px !important; cursor:pointer;text-decoration: none;"><?php echo __('Set up automatic Casso link', 'casso-wordpress-plugin'); ?></a>
								<?php echo __('or', 'casso-wordpress-plugin'); ?>
								<a style='text-decoration: none;' href='http://www.google.com' onclick="return showAuthorizarion()"><?php echo __('Click here to show manual configuration', 'casso-wordpress-plugin'); ?></a>


							</td>
						</tr>

						<tr style="<?php if ($this->oauth_settings['login_type'] != 1) echo "display: none;" ?>">
							<th scope="row"><?php echo __('Link Casso', 'casso-wordpress-plugin') ?></th>
							<td>
								<a type="button" href="<?php echo $this->getCurrentDomain() . '&oauth2_status=false' ?>" style="    border: 1px solid;padding: 7px;border-radius: 3px;text-decoration: none;"><?php echo __('Disconnect', 'casso-wordpress-plugin'); ?></a>
								<p class="description" style="margin-top: 10px;margin-left: 2px;">
									Connected as <?php echo get_bloginfo('name'); ?>
								</p>
							</td>
						</tr>









						<tr style=" <?php if (empty($settings['bank_transfer']['authorization_code'])) echo 'display: none;';
									else echo '';  ?>" id='link_authorization'>
							<th scope="row"><?php echo __('Authorization code', 'casso-wordpress-plugin') ?></th>
							<td>
								<input id='authorization_code' name="settings[bank_transfer][authorization_code]" style='min-width: 310px;' type="text" style='min-width: 200px;' value="<?php echo  $settings['bank_transfer']['authorization_code']; ?>"> <a href="https://casso.vn/plugin-ket-noi-ngan-hang/" onclick="window.open(this.href); return false;" onkeypress="window.open(this.href); return false;" style="margin-left: 5px;text-decoration: none;">Hướng dẫn lấy Casso Auth Code</a>
								<input id='authorization_code_force_delete' name="settings[bank_transfer][authorization_code_force_delete]" type="text" style='display: none;' value="<?php echo  $settings['bank_transfer']['authorization_code']; ?>">
								</br>
								<input type="submit" name="submit" id="submit" class="button button-primary" value="Liên kết" style="border: none; background-color: #056fca; color: white; border-radius: 5px; min-width: 110px !important;margin-top: 8px;">
								<button onclick="handleUnlink()" type="" name="submit" id="submit" class="button button-primary" value="" style="margin-left: 5px; border: none; background-color: #0000007a; color: white; border-radius: 5px; min-width: 95px !important;margin-top: 8px;">Hủy liên kết</button>
							</td>
						</tr>





						<tr>
							<th scope="row"><?php echo __('Connection Information', 'casso-wordpress-plugin') ?></th>
							<td>
								<button id='banks_list_user_button' onclick="showDetailBanks()" type="button" style="border: none; cursor:pointer; <?php if (empty($settings['bank_transfer']['authorization_code']) && $this->oauth_settings['login_type'] == 0) echo 'background-color: #af1818;';
																																					else echo 'background-color: #056fca;'  ?> color: white; border-radius: 5px; padding: 6px; min-width: 125px !important; margin-bottom: 10px;"><?php if (empty($settings['bank_transfer']['authorization_code']) && $this->oauth_settings['login_type'] == 0) echo __('No connection', 'casso-wordpress-plugin');
																																																																									else echo __('Show details', 'casso-wordpress-plugin'); ?></button>
								<div id='banks_list_user' style="padding: 20px; background-color: #bdbdbd; color: black; border: 1px solid; max-width: 500px; border-radius: 10px; display: none">
									<?php if (empty($settings['bank_transfer_accounts'])) __('No banks found', 'casso-wordpress-plugin'); ?>
									<ul style="<?php if (empty($settings['bank_transfer_accounts'])) echo "display: none" ?>">
										<li><b><?php echo __('Bussiness name', 'casso-wordpress-plugin') ?>: </b><?php echo $this->oauth_settings['business_id'] . ' | ' . $this->oauth_settings['business_name'] ?></li>
										<li><b><?php echo __('User', 'casso-wordpress-plugin') ?>: </b><?php echo $this->oauth_settings['user_id'] . ' | ' . $this->oauth_settings['email'] ?></li>
										<li><b><?php echo __('List of bank accounts', 'casso-wordpress-plugin') ?>: </b>
											<ul style="padding: 5px;">
												<?php
												$html = '';
												$i = 0;
												foreach ($settings['bank_transfer_accounts'] as $account) {
													$i++;
													$color = 'color: red;';
													if ($account['connect_status']) $color = 'color: #127b00;';
													$html .= '<li style="margin-left: 20px; margin-bottom: 2px;">' . $i . '. ' . $this->casso_get_list_banks[$account['bank_name']] . ' | ' . $account['account_number'] . ' | ' . $account['account_name'] . ' | <span style ="' . $color . '">' . __($this->casso_get_status[$account['connect_status']] == 0 ? $this->casso_get_status[$account['connect_status']] : $this->casso_get_status[$account['connect_status']][$account['plan_status']], 'casso-wordpress-plugin') . '</span></li>';
												}
												echo $html;
												?>
											</ul>
										</li>
										<li><b><?php echo __('Security Key', 'casso-wordpress-plugin') ?>: </b>******</li>
										<li><b><?php echo __('Webhook URL', 'casso-wordpress-plugin') ?>: </b><?php echo $this->get_webhook_url(); ?></li>
									</ul>
								</div>
							</td>
							<td></td>
						</tr>


						<tr style="<?php if (empty($settings['bank_transfer']['authorization_code']) && $this->oauth_settings['login_type'] == 0) echo 'display: none' ?>">
							<th scope="row"><?php echo __('Account on/off', 'casso-wordpress-plugin') ?></th>
							<td style="padding: inherit;">
								<ul>
									<?php
									$html = '';
									$i = 0;
									foreach ($settings['bank_transfer_accounts'] as $account) {
										$i++;
										$hasCheck = 'checked = checked';
										if (isset($account['is_show']) && $account['is_show'] == 'no') $hasCheck = '';
										$html .= '<li style="margin-left: 20px; margin-bottom: 8px;">
										
										<input name="settings[is_show_account][' . $account['account_number'] . ']" type="hidden" value="no">
										<input name="settings[is_show_account][' . $account['account_number'] . ']" type="checkbox" id="bank_transfer" value="yes"  ' . $hasCheck . '>
										
											
										' . $this->casso_get_list_banks[$account['bank_name']] . ' - ' . $account['account_number'] . ' - ' . ($account['account_name'] ?: __('Updating account name', 'casso-wordpress-plugin'));
									}
									echo $html;
									?>

								</ul>
							</td>
						</tr>







						<tr style="display: none;">
							<th scope="row"><?php echo __('Webhook URL', 'casso-wordpress-plugin') ?></th>
							<td>
								<input name="settings[bank_transfer][webhook]" readonly type="text" style='min-width: 400px;' value="<?php echo $this->get_webhook_url(); ?>">
							</td>
						</tr>
						<tr>
							<th scope="row"><?php echo __('Transaction prefix', 'casso-wordpress-plugin') ?></th>
							<td>
								<input name="settings[bank_transfer][transaction_prefix]" type="text" value="<?php echo  $settings['bank_transfer']['transaction_prefix']; ?>">
								<label for="bank_transfer" style="font-size: 13px; font-style: oblique;"><?php echo __('Maximum 15 characters, no spaces and no special characters. If contained, it will be deleted', 'casso-wordpress-plugin') ?></label>
							</td>
						</tr>
						<tr>
							<th scope="row"><?php echo __('Turn on Case Sensitivity', 'casso-wordpress-plugin') ?></th>
							<td>
								<input name="settings[bank_transfer][case_insensitive]" type="hidden" value="no">
								<input name="settings[bank_transfer][case_insensitive]" type="checkbox" id="bank_transfer" value="yes" <?php if ('yes' == $settings['bank_transfer']['case_insensitive']) echo 'checked="checked"';	?>>
								<label for="bank_transfer" style="font-size: 13px; font-style: oblique;"><?php echo __('Turn on Case Sensitivity', 'casso-wordpress-plugin') ?></label>
								<br />
							</td>
						</tr>

						<tr>
							<th scope="row"><?php echo __('Acceptance difference', 'casso-wordpress-plugin') ?></th>
							<td>
								<input name="settings[bank_transfer][acceptable_difference]" type="number" value="<?php echo  $settings['bank_transfer']['acceptable_difference']; ?>">
							</td>
						</tr>
						<tr>
							<th scope="row"><?php echo __('Status after full payment or balance:', 'casso-wordpress-plugin') ?></th>

							<td>
								<select name="settings[order_status][order_status_after_paid]" id="order_status_after_paid">
									<?php
									foreach ($this->casso_get_order_statuses_after_paid() as $key => $value) {
										if ($key == $settings['order_status']['order_status_after_paid'])
											echo '<option value="' . $key . '" selected>' . $value . '</option>';
										else echo '<option value="' . $key . '" >' . $value . '</option>';
									}
									?>
								</select>
							</td>
						<tr>
						<tr>
							<th scope="row"><?php echo  __('Status if payment is missing:', 'casso-wordpress-plugin') ?></th>

							<td>
								<select name="settings[order_status][order_status_after_underpaid]" id="order_status_after_underpaid">
									<?php
									foreach ($this->casso_get_order_statuses_after_underpaid() as $key => $value) {
										if ($key == $settings['order_status']['order_status_after_underpaid'])
											echo '<option value="' . $key . '" selected>' . $value . '</option>';
										else echo '<option value="' . $key . '">' . $value . '</option>';
									}
									?>
								</select>
							</td>
						<tr>

					</tbody>
				</table>

				<p class="submit">
					<input type="submit" name="submit" id="submit" class="button button-primary" value="Save Changes">
				</p>

			</form>
			<div id="casso-admin-footer" style="border: 1px dotted; padding: 5px;">
				<?php
				printf(
					__('Wanna get support or give feedback? Please <a href="%1$s">rate Casso</a> or post questions <a href="%2$s">in the forum</a>!', 'casso'),
					'https://wordpress.org/support/plugin/casso-tu-dong-xac-nhan-thanh-toan-chuyen-khoan-ngan-hang/reviews/',
					'https://wordpress.org/plugins/casso-tu-dong-xac-nhan-thanh-toan-chuyen-khoan-ngan-hang/'
				)
				?>
			</div>
		</div>
		<script type="text/javascript">
			function showAuthorizarion() {
				document.getElementById("link_authorization").style.display = "revert";
				return false;
			}

			function showDetailBanks() {
				var dots = document.getElementById("banks_list_user");
				var button = document.getElementById("banks_list_user_button");

				if (dots.style.display === "none") {
					dots.style.display = "block";
					button.innerHTML = "<?php echo __('Show less', 'casso-wordpress-plugin'); ?>"
				} else {
					dots.style.display = "none";
					button.innerHTML = "<?php if (empty($settings['bank_transfer']['authorization_code']) && $this->oauth_settings['login_type'] == 0) echo __('No connection', 'casso-wordpress-plugin');
										else echo __('Show details', 'casso-wordpress-plugin'); ?>";
				}


			}

			function handleUnlink() {
				document.getElementById("authorization_code").value = "";
			}
		</script>
		<!-- #wrap ->
        <?php
	}
	public function disconnectOAuth2()
	{
		if ($_REQUEST['oauth2_status'] == 'true' || $this->oauth_settings['login_type'] == 0) return;
		$response_token = CassoPayment::casso_oauth_get_token($this->oauth_settings['refresh_token'], $this->oauth_settings['login_type']);
		$response_token = json_decode($response_token);

		CassoPayment::casso_oauth_force_delete_all_webhook($response_token->access_token, $this->settings['bank_transfer']['webhook'], $this->oauth_settings['refresh_token'], $this->oauth_settings['login_type']);
		$accounts_emp = array();
		$settings['bank_transfer_accounts'] = $accounts_emp;
		update_option('casso_oauth', $accounts_emp);
		update_option('casso', $settings);
		header('Location: '. parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH) . '?page=casso');
		die();
	}
	static function get_webhook_url()
	{
		return WC()->api_request_url(CassoPayment::$webhook_route);
	}
	public function casso_get_order_statuses_after_paid()
	{
		$wooDefaultStatuses = array(
			"wc-pending",
			"wc-processing",
			"wc-on-hold",
			// "wc-completed",
			"wc-cancelled",
			"wc-refunded",
			"wc-failed",
			// "wc-paid",
			"wc-underpaid"
		);
		$statuses =  wc_get_order_statuses();
		$statuses['wc-default'] = __('Default', 'casso-wordpress-plugin');
		for ($i = 0; $i < count($wooDefaultStatuses); $i++) {
			$statusName = $wooDefaultStatuses[$i];
			if (isset($statuses[$statusName])) {
				unset($statuses[$statusName]);
			}
		}
		return $statuses;
	}
	public function getCurrentDomain()
	{
		$uri = $_SERVER['REQUEST_URI'];
		echo $uri; // Outputs: URI

		$protocol = ((!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";

		$url = $protocol . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
		echo $url; // Outputs: Full URL

		$query = $_SERVER['QUERY_STRING'];
		echo $query; // Outputs: Query String
	}
	public function casso_get_order_statuses_after_underpaid()
	{
		$wooDefaultStatuses = array(
			"wc-pending",
			// "wc-processing",
			"wc-on-hold",
			"wc-completed",
			"wc-cancelled",
			"wc-refunded",
			"wc-failed",
			"wc-paid",
			// "wc-underpaid"
		);
		$statuses =  wc_get_order_statuses();
		$statuses['wc-default'] =  __('Default', 'casso-wordpress-plugin');
		for ($i = 0; $i < count($wooDefaultStatuses); $i++) {
			$statusName = $wooDefaultStatuses[$i];
			if (isset($statuses[$statusName])) {
				unset($statuses[$statusName]);
			}
		}
		return $statuses;
	}

	private function casso_oauth_process_webhook($settings)
	{
		$AuthCode = $settings['bank_transfer']['authorization_code'];
		if (empty($AuthCode) && $this->oauth_settings['login_type'] == 0) {
			$accounts_emp = array();
			$settings['bank_transfer_accounts'] = $accounts_emp;
			update_option('casso_oauth', $accounts_emp);
			update_option('casso', $settings);
			if (!empty($settings['bank_transfer']['authorization_code_force_delete'])) {
				$response_token_temp = null;

				if (strlen($settings['bank_transfer']['authorization_code_force_delete']) <= 36) {
					$response_token_temp = CassoPayment::casso_oauth_get_token($settings['bank_transfer']['authorization_code_force_delete']);
					if (empty($response_token_temp)) {
						return;
					}
					$response_token_temp = json_decode($response_token_temp);
				}
				$token_delete = null;

				if (!empty($response_token_temp->access_token)) {
					$token_delete = $response_token_temp->access_token;
				}
				CassoPayment::casso_oauth_force_delete_all_webhook($token_delete, $settings['bank_transfer']['webhook'], $settings['bank_transfer']['authorization_code_force_delete']);
			}
			header("Refresh:0");
			return;
		}
		$loginType = false;
		if ($this->oauth_settings['login_type'] == 1) {
			$loginType = true;
			$AuthCode = $this->oauth_settings['refresh_token'];
		}
		$response_token = null;

		if (strlen($AuthCode) <= 36) {
			$response_token = CassoPayment::casso_oauth_get_token($AuthCode, $loginType);
			if (empty($response_token)) {
				return '<div class="error notice"><p><strong>' .
					__('Webhook creation failed', 'casso') .
					'</p></strong></div>';
			}

			//get token from casso
			$response_token = json_decode($response_token);
		}
		if (empty($response_token->access_token)) {
			$this->oauth_settings['refresh_token'] = null;
			$this->oauth_settings['access_token'] = null;
			$this->oauth_settings['expires_at'] = 0;
		} else {
			$this->oauth_settings['refresh_token'] = $loginType ? $AuthCode : $response_token->refresh_token;
			$this->oauth_settings['access_token'] = $response_token->access_token;
			$this->oauth_settings['expires_at'] = time() + $response_token->expires_in;
		}
		// delete all old webhook
		CassoPayment::casso_oauth_force_delete_all_webhook($this->oauth_settings['access_token'], $settings['bank_transfer']['webhook'], $AuthCode, $loginType);
		// register new webhook
		$response_create = CassoPayment::casso_oauth_create_webhook($this->oauth_settings['access_token'], $settings, $loginType);

		if (empty($response_create)) {
			return '<div class="error notice"><p><strong>' .
				__('Webhook creation failed', 'casso') .
				'</p></strong></div>';
		}
		$response_create = json_decode($response_create);
		$this->oauth_settings['webhook_id'] = $response_create->data->id;


		$response_user_info = CassoPayment::casso_oauth_get_user_infor($this->oauth_settings['access_token'], $AuthCode, $loginType);

		if (empty($response_user_info)) {
			return '<div class="error notice"><p><strong>' .
				__('Webhook creation failed', 'casso') .
				'</p></strong></div>';
		}
		$response_user_info = json_decode($response_user_info);
		$this->oauth_settings['user_id'] = $response_user_info->data->user->id;
		$this->oauth_settings['email'] = $response_user_info->data->user->email;
		$this->oauth_settings['business_id'] = $response_user_info->data->business->id;
		$this->oauth_settings['business_name'] = $response_user_info->data->business->name;
		$accounts = array();
		foreach ($response_user_info->data->bankAccs as $value) {
			$account = array(
				"account_name" => $value->bankAccountName,
				"account_number" => $value->bankSubAccId,
				"bank_name" => $this->casso_get_list_bin[$value->bank->bin ?: $value->bank->codeName],
				"bin" => $value->bank->bin ?: null,
				"connect_status" => $value->connectStatus,
				"plan_status" => $value->planStatus,
				"is_show" => empty($settings['is_show_account'][$value->bankSubAccId]) ? 'yes' : $settings['is_show_account'][$value->bankSubAccId]
			);
			$accounts[] = $account;
		}
		$settings['bank_transfer_accounts'] = $accounts;
		update_option('casso', $settings);
		update_option('casso_oauth', $this->oauth_settings);
		return '<div class="updated notice"><p><strong>' .
			__('Successful webhook registration', 'casso') .
			'</p></strong></div>';
	}
	public function clean_prefix($string)
	{
		$string = str_replace(' ', '', $string); // Replaces all spaces with hyphens.
		if (strlen($string) > 15) {
			$string = substr($string, 0, 15);
		}
		return preg_replace('/[^A-Za-z0-9\-]/', '', $string); // Removes special chars.
	}
}
